import unittest
from parser import parse, parse_config
import json
import os

class TestParser(unittest.TestCase):

    def test_parse_sources(self):
        config = {
            "mods_dir": "mods",
            "mods": [
                {
                    "source": "GITHUB",
                    "owner": "snallapa",
                    "repo": "scentfindermod"
                }
            ]
        }
        sources = parse(config)
        self.assertEqual(1, len(sources))
        self.assertEqual("GithubSource", sources[0].__class__.__name__)

    def test_parse_fails_missing_dir(self):
        config = {}
        self.assertRaises(Exception, parse, config)

    def test_parse_fails_invalid_source(self):
        config = {
            "mods_dir": "mods",
            "mods": [
                {
                    "source": "foo",
                }
            ]
        }
        self.assertRaises(Exception, parse, config)

    def test_parse_reads_file(self):
        config = {
            "mods_dir": "mods",
            "mods": [
                {
                    "source": "GITHUB",
                    "owner": "snallapa",
                    "repo": "scentfindermod"
                }
            ]
        }
        with open('config.json', 'w') as outfile:
            json.dump(config, outfile)
        
        def cleanup():
            os.remove("config.json")
        self.addCleanup(cleanup)
        sources = parse_config()
        self.assertEqual(1, len(sources))
        self.assertEqual("GithubSource", sources[0].__class__.__name__)



from .base import SourceTest
from sources.github import GithubSource
import os

class TestGithub(SourceTest):

    def test_download(self):
        g = GithubSource("snallapa", "scentfindermod", self.TEST_MODS_FOLDER)
        g.download()
        self.assertTrue(os.path.exists(f"{self.TEST_MODS_FOLDER}/scentfindermod-1.5.jar"))

    def test_parse(self):
        json = {
            "owner": "snallapa",
            "repo": "scentfindermod"
        }
        g = GithubSource.parse(json, self.TEST_MODS_FOLDER)
        g.download()
        self.assertTrue(os.path.exists(f"{self.TEST_MODS_FOLDER}/scentfindermod-1.5.jar"))


from .parser import parse_config
import argparse

def main():
    parser = argparse.ArgumentParser(description='config path')
    parser.add_argument('--config', metavar='config_path', type=str, default="config.json")
    args = parser.parse_args()
    sources = parse_config(args.config)
    for source in sources:
        source.download()


from abc import ABC, abstractmethod
import requests
from tqdm import tqdm
from mod_updater.util.utils import TermColors
import os

class DownloadSource(ABC):

    def download_file(self, response, file_path):
        total_size_in_bytes= int(response.headers.get('content-length', 0))
        block_size = 1024
        progress_bar = tqdm(total=total_size_in_bytes, unit='iB', unit_scale=True)
        os.makedirs(os.path.dirname(file_path), exist_ok=True)
        with open(file_path, 'wb') as file:
            for data in response.iter_content(block_size):
                progress_bar.update(len(data))
                file.write(data)
        progress_bar.close()
        if total_size_in_bytes != 0 and progress_bar.n != total_size_in_bytes:
            print(f"{TermColors.FAIL}ERROR, something went wrong{TermColors.ENDC}")

    @abstractmethod
    def download(self):
        raise NotImplementedError()

    @classmethod
    @abstractmethod
    def parse(cls, json):
        raise NotImplementedError()



import unittest
import os
import shutil

class SourceTest(unittest.TestCase):
    TEST_MODS_FOLDER = "test_mods"

    def tearDown(self):
        if os.path.exists(self.TEST_MODS_FOLDER) and os.path.isdir(self.TEST_MODS_FOLDER):
            shutil.rmtree(self.TEST_MODS_FOLDER)


from .base import DownloadSource
import requests

class GithubSource(DownloadSource):
    REQUIRED_ARGS = ["owner", "repo"]
    
    def __init__(self, owner, repo, path):
        self._releases_url = f"https://api.github.com/repos/{owner}/{repo}/releases/latest"
        self._path = path

    def download(self):
        r = requests.get(self._releases_url)
        download_url = r.json()["assets"][0]["browser_download_url"]
        name = r.json()["assets"][0]["name"]
        download_response = requests.get(download_url, stream=True)
        self.download_file(download_response, f"{self._path}/{name}")

    @classmethod
    def parse(cls, json, path):
        for arg in cls.REQUIRED_ARGS:
            if arg not in json:
                raise Exception("Missing required arg ", arg, "in ", json)
        owner = json["owner"]
        repo = json["repo"]
        return cls(owner, repo, path)



import json
import os
from mod_updater.sources.github import GithubSource

source_type_mapping = {
    "GITHUB": GithubSource
}

def parse(config):
    if "mods_dir" not in config:
        raise Exception("Required mods_dir, this is the path to the mods directory")
    mods_path = config["mods_dir"]
    mod_sources = config["mods"]
    sources = []
    for s in mod_sources:
        source_type = s["source"]
        if source_type not in source_type_mapping:
            raise Exception("Invalid source type ", source_type)
        source_class = source_type_mapping[source_type]
        sources.append(source_class.parse(s, mods_path))
    return sources 

def parse_config(config_file="config.json"):
    if not os.path.exists(config_file):
        raise Exception("Cannot find config file ", config_file)
    with open(config_file) as f:
        config = json.load(f)
    return parse(config)





import io
import os

from setuptools import find_packages, setup

NAME = 'mod_updater'
DESCRIPTION = 'Automatically update forge minecraft mods'
URL = 'https://github.com/snallapa/mod_updater'
EMAIL = 'sahith.reddy@gmail.com'
AUTHOR = 'Sahith Nallapareddy'
REQUIRES_PYTHON = '>=3.6.0'
VERSION = '0.2.0'

REQUIRED = [
    'requests', 'tqdm'
]

here = os.path.abspath(os.path.dirname(__file__))

try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    entry_points={
        'console_scripts': ['mod_updater=mod_updater.mod_updater:main'],
    },
    install_requires=REQUIRED,
    extras_require={},
    include_package_data=True,
    license='MIT',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        "Operating System :: OS Independent"
    ],
)